% MATLAB code: Appendix 11.A (Table 11.6)
% File: Critical_values_LR.m
%
% Multivariate LR-test statistic asymptotic critical values
%
% Note 1:
%   The M-functions "vgxet" and "vgxsim" require a licence of the
%   MATLAB Econometrics_Toolbox. 
%   If you do not have the licence or it has expired, simulate
%   the bivariate VTAR time series in the "old-fashioned" way; see below.
%
% Note 2:
%   As an example we fixed p = 2, m = 2 (fixed throughout) and alpha = 0.01. 
%   Make appropriate changes to the code for different values of these
%   parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
A0    = [1.2, -0.5, 0.7];
Sigma = [1, 0.7, 0.5; 0.7, 1, 0.7; 0.5, 0.7, 1];

% vgxet = Set VARMAX model specification parameters (blocked part = %%%)
% Example for lag order p = 5
%%% Spec = vgxset('a', [1.2, -0.5, 0.7], ...
%%% 'AR',{[0, 0, 0;  0, 0,  0; 0,  0,  0], ...
%%%       [0, 0, 0;  0, 0,  0; 0,  0,  0],...
%%%       [0, 0, 0;  0, 0,  0; 0,  0,  0], ...
%%%       [0, 0, 0;  0, 0,  0; 0,  0,  0], ...
%%%       [0, 0, 0;  0, 0,  0; 0,  0,  0]}, ... 
%%% 'Q',[1, 0.7, 0.5; 0.7, 1, 0.7; 0.5, 0.7, 1] );

sig = 0.01;                  % significance level
mu  = [0, 0, 0];
M   = 2000;                  % for averaging percentile points

p   = 2;                     % order of VAR(p) CHANGE
m   = 2;                     % dimension, bivariate model (fixed throughout)
nr  = 1000;

for w = 1:7                  % Big loop for seven values of r0
  low = 0.1+0.05*(w-1); 
  for u = 1:M                % Big loop for averaging percentile points

    R  = mvnrnd(mu,Sigma,1000+p); % multivariate normal random deviates
    nr = length(R)-p;
    Y  = zeros(nr,m);

    for i=p+1:nr                  
      Y(i,1) = A0(1)+R(i,1);
      Y(i,2) = A0(2)+R(i,2);        
     end

%   Econometrics-Toolbox (blocked part = %%%)
%   vgxsim = Simulate VARMAX model responses
%%% Y = vgxsim(Spec,nr);

    Z    = Y(p+1:nr,:);    % remove p zeros from first row
    high = (1-low);

    pset = [low,high];
    perc = quantile(Z(:,1),pset);  % else take linspace(perc(1),perc(2),10)
    rl   = perc(1);                % LOWER percentile
    rh   = perc(2);                % HIGHER   ,,

    iT = length(Z);
    for i=1:(iT-p)
       if(Z(i,1)<= rl)        % 1st variable,  LOW percentile
         indl(i,1) = 1;       % dim = (iT-p)*1, assuming d = 1, else CHANGE
      else
         indl(i,1) = 0;
      end
    end

    for i=1:(iT-p)
      if(Z(i,1)<= rh)         % 1st variable,  HIGH percentile
         indh(i,1) = 1;       % dim = (iT-p)*1, assuming d = 1, else CHANGE
      else
         indh(i,1) = 0;
      end
    end

    for i=1:p
      mX(:,:,i) = Z(p-i+1:iT-i,:);          % matrix X
    end

    for i=1:(iT-p)
      mXind1l(i,:) = indl(i,1)*mX(i,:,1);   % dim = (iT-p)*m  LOW, p=1
      mXind1h(i,:) = indh(i,1)*mX(i,:,1);   % dim = (iT-p)*m  HIGH   
    
      mXind2l(i,:) = indl(i,1)*mX(i,:,2);   % CHANGE for different values of p=2, LOW
      mXind2h(i,:) = indh(i,1)*mX(i,:,2);   % dim = (iT-p)*m  HIGH    
 
%%%   mXind3l(i,:) = indl(i,1)*mX(i,:,3);   % CHANGE for different values of p=3, LOW
%%%   mXind3h(i,:) = indh(i,1)*mX(i,:,3);   % dim = (iT-p)*m  HIGH    
 
%%%   mXind4l(i,:) = indl(i,1)*mX(i,:,4);   % CHANGE for different values of p=4, LOW
%%%   mXind4h(i,:) = indh(i,1)*mX(i,:,4);   % dim = (iT-p)*m  HIGH    

%%%   mXind5l(i,:) = indl(i,1)*mX(i,:,5);   % CHANGE for different values of p=5, LOW
%%%   mXind5h(i,:) = indh(i,1)*mX(i,:,5);   % dim = (iT-p)*m  HIGH    
    end
    
    Yrl = [indl,mXind1l]; % matrix Yr LOW,  dim = (iT-p)*(mp+1)
    Yrh = [indh,mXind1h]; % matrix Yr HIGH, dim = (iT-p)*(mp+1)

    Yrl = [indl,mXind1l,mXind2l]; % matrix Yr LOW,  CHANGE p=2, dim = (iT-p)*(mp+1)
    Yrh = [indh,mXind1h,mXind2h]; % matrix Yr HIGH, CHANGE p=2, dim = (iT-p)*(mp+1)

%%  Yrl = [indl,mXind1l,mXind2l,mXind3l]; % matrix Yr LOW,  CHANGE p=3, dim = (iT-p)*(mp+1)
%%  Yrh = [indh,mXind1h,mXind2h,mXind3h]; % matrix Yr HIGH, CHANGE p=3, dim = (iT-p)*(mp+1)

%%  Yrl = [indl,mXind1l,mXind2l,mXind3l,mXind4l]; % matrix Yr LOW, CHANGE p=4, dim = (iT-p)*(mp+1)
%%  Yrh = [indh,mXind1h,mXind2h,mXind3h,mXind4h; % matrix Yr HIGH, CHANGE p=4, dim = (iT-p)*(mp+1)
       
%%  Yrl = [indl,mXind1l,mXind2l,mXind3l,mXind4l,mXind5l]; % matrix Yr LOW,  CHANGE p=5, dim = (iT-p)*(mp+1)
%%  Yrh = [indh,mXind1h,mXind2h,mXind3h,mXind4h,mXind5h]; % matrix Yr HIGH, CHANGE p=5, dim = (iT-p)*(mp+1)

    mXone = ones(iT-p,1);
    
    mXtot = [mXone,mX(:,:,1)]; % CHANGE p=1               
    mXtot = [mXone,mX(:,:,1),mX(:,:,2)]; % CHANGE p=2
%%% mXtot = [mXone,mX(:,:,1),mX(:,:,2),mX(:,:,3)]; % CHANGE p=3
%%% mXtot = [mXone,mX(:,:,1),mX(:,:,2),mX(:,:,3),mX(:,:,4)]; % CHANGE p=4
%%% mXtot = [mXone,mX(:,:,1),mX(:,:,2),mX(:,:,3),mX(:,:,4),mX(:,:,5)]; % CHANGE p=5

    Srl = Yrl'*Yrl/iT;                % (pm+1)*(pm+1) estimate of Sigma_r, LOW
    Srh = Yrh'*Yrh/iT;                % (pm+1)*(pm+1) estimate of Sigma_r, HIGH  

    Sx = mXtot'*mXtot/iT;             % (pm+1)*(pm+1) estimate of Sigma_x (X matrix)

    [uUl, sUl, vUl] = svd(Srl);       % singular value decomposition LOW
    [uUh, sUh, vUh] = svd(Srh);       % singular value decomposition HIGH

    Sr5l = vUl * inv(sqrt(sUl))*vUl'; % Sigma_r^{-1/2}: (pm+1)*(pm+1) matrix LOW
    Sr5h = vUh * inv(sqrt(sUh))*vUh'; % Sigma_r^{-1/2}: (pm+1)*(pm+1) matrix HIGH

    [uR, sR, vR] = svd(Sx);           % singular value decomposition X matrix
    Sx5 = vR*inv(sqrt(sR))*vR';       % Sigma_x^{-1/2}: m*m matrix

    rootl = eig(Sx5*Srl*Sx5);         % eigenvalues of Sigma_x^{-1/2}*Sigma_r^{-1/2}*Sigma_x^{-1/2} LOW
    rooth = eig(Sx5*Srh*Sx5);         % eigenvalues of Sigma_x^{-1/2}*Sigma_r^{-1/2}*Sigma_x^{-1/2} HIGH

    %%%  Start computing asymptotic percentile points: zz
    for i=1:(p*m+1)
      tpl(i)   = 0.5*log(rootl(i)/(1-rootl(i)));
      tph(i)   = 0.5*log(rooth(i)/(1-rooth(i))); 
      tpdif(i) = tph(i)-tpl(i);
    end

    temp = sum(tpdif);        % sum^{p(m+1)}(tpl - tph)
    func =@(x)1-exp(-2*chi2pdf(x,m*(p*m+1))*(x/(p*m+1)-m)*temp)-sig;

    zz(u) = fzero(func,55);   % find roots of continuous function of one variable

  end  % end u=1:M, big loop to obtain percentile points for averaging

  totdisp = [sig,p,m,low,mean(zz)];
  disp(totdisp)

end  % end Big loop w = 1:7, seven values of r0